using System;
using System.Collections.Generic;
using System.Text;
using javax.naming;
using javax.rmi;
using java.util;
using java.lang;
using examples.ejb.ejb20.basic.beanManaged;
using javax.ejb;

namespace WebLogic10Example
{
    class Program 
    {
        static string url;
        static string hostName = "";     // edit host name or IP address
        static string portNumber = "";  // edit with valid WebLogic t3 port, usually '7001'
        static string login = "";            // edit with a login, if required by WebLogic implementation
        static string passWord = "";     // edit with pass wprd, if required by WebLogic implementation     

        private AccountHome home;
        
        public Program()  
        {
            home = lookupHome();
        }
  
        static void Main(string[] args)  
        {
            Console.WriteLine("\nBeginning beanManaged.Client...\n");
            
            Program client = null;
            Program.url= "t3://" + hostName + ":" + portNumber;
            try 
            {
                  client = new Program();
                  client.example();
            } 
            catch (NamingException ne) 
            {
                  Console.WriteLine("Unable to look up the beans home: " + ne.getMessage());
                  throw ne;
            } 
            catch (java.lang.Exception e) 
            {
                  Console.WriteLine("There was an exception while creating and using the Accounts.");
                  Console.WriteLine("This indicates that there was a problem communicating with the server: "+e);
                  throw e;
            }
            Console.WriteLine("\nEnd beanManaged.Client...\n");
        }
        
        
        public void example()
        {
            int numBeans = 20;
            Account [] accounts = new Account [numBeans];

            // Create 20 accounts
            for (int i=0; i<numBeans; i++) 
            {
                accounts [i] = findOrCreateAccount("ID: "+i, i * 1000);
            }

            // print out the account balances
            for (int i=0; i<numBeans; i++) 
            {
                  Console.WriteLine("Account: :"+accounts[i].getPrimaryKey()+
                    " has a balance of "+accounts[i].balance());
            }

            // find all accounts with a balance > 5000
            findBigAccounts(5000.0);

            // Remove our accounts
            Console.WriteLine("Removing beans...");
            for (int i=0; i<numBeans; i++) 
            {
                  accounts[i].remove();
            }
        }
         
        private void findBigAccounts(double balanceGreaterThan)
        {
            Console.WriteLine("\nQuerying for accounts with a balance greater than " + balanceGreaterThan + "...");

            Collection col = home.findBigAccounts(balanceGreaterThan);

            if(col.isEmpty()) 
            {
                Console.WriteLine("No accounts were found with a balance greater that " + balanceGreaterThan);
            }

            Iterator it = col.iterator();
            while (it.hasNext()) 
            {
                Account accountGT = (Account) PortableRemoteObject.narrow(it.next()
                    , java.lang.Class.forName("examples.ejb.ejb20.basic.beanManaged.Account"));
                Console.WriteLine("Account " + accountGT.getPrimaryKey() + "; balance is $" + accountGT.balance());
            }
        }

     
        private Account findOrCreateAccount(string id, double balance)
        {
            try 
            {
                Console.WriteLine("Creating account with id: "+id);
                // create account with account id and balance.
                java.lang.Object obj = (java.lang.Object)home.create(id, balance);
                return (Account) PortableRemoteObject.narrow(obj
                    , java.lang.Class.forName("examples.ejb.ejb20.basic.beanManaged.Account"));
            } 
            catch (CreateException cex) 
            {
                // account id already exists.
                return (Account) PortableRemoteObject.narrow((java.lang.Object)home.findByPrimaryKey(id)
                    , java.lang.Class.forName("examples.ejb.ejb20.basic.beanManaged.Account"));                                          
            }
        }

  
        private Account createAccount(string id, double balance)
        {
            Console.WriteLine("Creating account " + id + " with a balance of " + balance + "...");
            Account ac = (Account) PortableRemoteObject.narrow((java.lang.Object)home.create(id, balance)
                , java.lang.Class.forName("examples.ejb.ejb20.basic.beanManaged.Account"));      
            Console.WriteLine("Account " + id + " successfully created");
            return ac;
        }

  
        private AccountHome lookupHome()
        {
            Context ctx = getInitialContext();
            try 
            {
                java.lang.Object home = ctx.lookup("ejb20-beanManaged-AccountHome");
                return (AccountHome) PortableRemoteObject.narrow(home
                    , java.lang.Class.forName("examples.ejb.ejb20.basic.beanManaged.AccountHome"));

            } 
            catch (NamingException ne) 
            {
                Console.WriteLine("The client was unable to lookup the EJBHome.  Please make sure " +
                "that you have deployed the ejb with the JNDI name " +
                "ejb20-beanManaged-AccountHome on the WebLogic server at "+url);
                throw ne;
            }
        }

  
        private Context getInitialContext() 
        {
            try 
            {
                Properties h = new Properties();
                h.put((java.lang.JavaString)ContextConstants.INITIAL_CONTEXT_FACTORY
                    , (java.lang.JavaString)"weblogic.jndi.WLInitialContextFactory");

                h.put((java.lang.JavaString)ContextConstants.PROVIDER_URL
                    , (java.lang.JavaString)url);

                if ( !string.IsNullOrEmpty(login) )
                {
                    h.put((java.lang.JavaString)ContextConstants.SECURITY_AUTHENTICATION
                        , (java.lang.JavaString)"simple");
                    h.put(new java.lang.JavaString(ContextConstants.SECURITY_PRINCIPAL)
                        , (java.lang.JavaString)login);
                    h.put(new java.lang.JavaString(ContextConstants.SECURITY_CREDENTIALS)
                        , (java.lang.JavaString)passWord);
                }

                return new InitialContext(h);

            } 
                catch (NamingException ne) {
                Console.WriteLine("We were unable to get a connection to the WebLogic server at "+url);
                Console.WriteLine("Please make sure that the server is running.");
                throw ne;
            }
        }

}


}
